import java.util.LinkedList;

class Node {
  private String cityname;          // name of city at this node
  private LinkedList<Node> getsTo;  // edges from this Node
    
  // constructor only takes the cityname as an argument,
  //   initializing the getsTo list internally
  Node(String cityname) {
    this.cityname = cityname;
    this.getsTo = new LinkedList<Node>();
  }

  // method for printing out the contents of a Node object
  public String toString () {
    return this.cityname;
  }

  // adds an edge from this node to the given toNode
  public void addEdge(Node toNode) {
    this.getsTo.add(toNode);
  }
  
  // produces a route from this node to the to Node, if one exists
  // The produced route should not include the same city more than once
  LinkedList<Node> findRoute(Node to, LinkedList<Node> visited) {
    // if have tried this node before, there is no route
    if (visited.contains(this)) {
      return new LinkedList<Node>();
    } 
    // if this is the node we are searching for, have a route with one node
    else if (this.equals(to)) {
      LinkedList<Node> newRoute = new LinkedList<Node>();
      newRoute.addFirst(this);
      return newRoute;
    }
    // search for route through the getsTo nodes
    else { 
      visited.add(this);
      return this.findRouteEdges(to,this,visited);
    }
  }
  
  // checks for a route using the edges out of getsTo
  LinkedList<Node> findRouteEdges(Node to, Node edgeSource, LinkedList<Node> visited) {
    for (Node c : this.getsTo) {
      LinkedList<Node> cRoute = c.findRoute(to,visited);
      if (cRoute.size() > 0) {
        cRoute.addFirst(edgeSource);
        return cRoute;
      }
    }
    return new LinkedList<Node>(); // no route from children, so return empty list
  }
}